import cors from "cors";
import express from "express";
import multer from "multer";
import {createFFmpeg} from "@ffmpeg/ffmpeg";

const ffmpegInstance = createFFmpeg({log: true});
let ffmpegLoadingPromise = ffmpegInstance.load();

async function getFFmpeg() {
    await ffmpegLoadingPromise;
    return ffmpegInstance;
}

const app = express();
const port = 3000;

const uploadMiddleware = multer({
    storage: multer.memoryStorage(),
    limits: {fileSize: 150 * 1024 * 1024}, // 150MB
    fileFilter(req, file, callback) {
        if (['video/mp4', 'video/x-matroska'].includes(file.mimetype)) {
            return callback(null, true);
        }
        return callback(new Error('Invalid file type, only mp4 and mkv are allowed'));
    }
})

app.use(cors());

async function convertVideo(inputVideoName: string, inputVideoData: Buffer, outputVideoName: string) {
    try {
        const ffmpeg = await getFFmpeg();

        ffmpeg.FS('writeFile', inputVideoName, inputVideoData);

        await ffmpeg.run(
            '-i', inputVideoName,
            '-vcodec', 'libx264',
            '-crf', '22',
            '-preset', 'slow',
            '-vf', 'format=yuv420p',
            '-c:a', 'aac',
            '-b:a', '160k',
            outputVideoName
        )

        let outputVideoData = ffmpeg.FS('readFile', outputVideoName);
        ffmpeg.FS('unlink', inputVideoName);
        ffmpeg.FS('unlink', outputVideoName);
        return outputVideoData;
    } catch (e) {
        console.error(e);
        throw e;
    }
}

app.post('/convert', uploadMiddleware.single('video'), async (req, res) => {
    const inputVideoData = req.file?.buffer;
    const inputVideoName = 'input';
    const outputVideoName = 'output.mp4';
    let outputVideoData = null;

    if (!inputVideoData) {
        res.status(400).send('No video file provided');
        return;
    }

    try {
        outputVideoData = await convertVideo(inputVideoName, inputVideoData, outputVideoName);
    } catch (e: any) {
        res.status(500).send('Error converting video : ' + e.message);
        return;
    }

    res.writeHead(200, {
        'Content-Type': 'video/mp4',
        'Content-Disposition': `attachment; filename="${outputVideoName}"`,
        'Content-Length': outputVideoData.length
    });
    res.end(outputVideoData.buffer);
});

app.listen(port, () => {
    console.info(`Server running on port ${port}`);
});

